<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\Customer;
use App\Models\Faq;
use App\Models\Setting;
use GuzzleHttp\Client;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Route;
use Illuminate\Support\Str;

class FaqController extends Controller
{
    public function index(Request $request) {
        $records = Faq::select('id','question','answer','status')->orderBy('created_at','DESC')->paginate($this->defaultPaginate);
        return view('admin.faq.index',['records' => $records]);
    }

    public function add() {
        return view('admin.faq.add');
    }

    protected function validateData ($request) {
//        dd(Route::currentRouteName());

         $validationArray = [];
        if (Route::currentRouteName() == 'faq.store') {
            $validationArray = [
                'question' => ['required'],
                'answer' => ['required'],
                'status' => ['required'],
            ];
        }
        

        $this->validate($request,$validationArray);
    }

    public function store(Request $request) {


        $this->validateData($request);
        $data = new Faq($request->only('question','answer','status'));
        $data->save();

        return redirect(route('faq'))->with('success','Faq Created Successfully');
    }

    public function edit($id) {

        return view('admin.faq.edit',[
            'data' => Faq::findOrFail($id),
        ]);
    }

    public function update(Request $request,$id) {

        $this->validateData($request);
        $data = Faq::findOrFail($id);
        $data->fill($request->only('question','answer','status'))->save();

        return redirect(route('faq'))->with('success','Faq Updated Successfully');
    }

    public function delete($id) {
        if(! $data = Faq::whereId($id)->first()) {
            return redirect()->back()->with('error', 'Something went wrong');
        }

        $data->delete();
        return redirect(route('faq'))->with('success', 'Faq  Deleted Successfully');
    }
}
